from discovery.data_matching.matching_methods.match_column_name_lcs import MatchColumnNamesLCS
from discovery.data_matching.matching_methods.match_column_name_levenshtein import MatchColumnNamesLevenshtein
from discovery.data_matching.matching_methods.match_column_name_wordnet import MatchColumnNamesWordnet
from discovery.data_matching.matching_methods.match_data_dynamic_time_warping import MatchDataDynamicTimeWarping
from discovery.data_matching.matching_methods.match_data_pearson_coefficient import MatchDataPearsonCoefficient
from discovery.data_matching.matching_methods.match_data_two_sample_t_test import MatchDataTwoSampleTTest
from discovery.data_matching.matching_methods.match_identical_rows import MatchIdenticalRows
from discovery.data_matching.matching_methods.match_identical_column_names import MatchIdenticalColumnNames